import 'package:flutter/material.dart';
import 'package:flutter_gadgets_2/flutter_gadgets_2.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:provider/provider.dart';

import '../utils.dart';

Type _typeOf<T>() => T;

void main() {
  testWidgets('ViewModelGadget with subscribeToViewModel test', (tester) async {
    await tester.pumpWidget(
      AppGadget(
        child: ViewModelGadget(
          viewModel: MockViewModel(),
          subscribeToViewModel: true,
          init: (MockViewModel viewModel) => viewModel.state = 10,
          builder: (_, __) => Container(),
        ),
      ),
    );

    expect(
      find.byType(_typeOf<SubscriberGadget<MockViewModel>>()),
      findsOneWidget,
    );

    final viewModel = Provider.of<ObservableModel>(
      tester.element(find.byType(Container)),
      listen: false,
    ).getValue<MockViewModel>();
    expect(viewModel, isNotNull);
    expect(viewModel!.state, 10);

    await tester.pumpWidget(Container());
  });

  testWidgets('ViewModelGadget without subscribeToViewModel test',
      (tester) async {
    await tester.pumpWidget(
      AppGadget(
        child: ViewModelGadget(
          viewModel: MockViewModel(),
          init: (MockViewModel viewModel) => viewModel.state = 2,
          builder: (_, __) => Container(),
        ),
      ),
    );

    expect(
      find.byType(_typeOf<SubscriberGadget<MockViewModel>>()),
      findsNothing,
    );

    final viewModel = Provider.of<ObservableModel>(
      tester.element(find.byType(Container)),
      listen: false,
    ).getValue<MockViewModel>();
    expect(viewModel, isNotNull);
    expect(viewModel!.state, 2);
  });
}
